import axios from 'axios';
import fs from 'fs';
import _ from 'lodash';

let current = parseInt(process.env.CURRENT) || 0;
const max = parseInt(process.env.MAX) || 26 * 26 * 26 * 26 - 1;
const parallel = parseInt(process.env.PARALLEL) || 10;
const urlPrefix = process.env.URL_PREFIX;

function toCode(n: number) {
  let code = '';
  for (let i = 0; i < 4; i++) {
    code = String.fromCharCode((n % 26) + 'a'.charCodeAt(0)) + code;
    n = Math.floor(n / 26);
  }
  return code;
}

const useCode = async (n: number) => {
  const code = toCode(n);
  console.log(`Trying ${n}: ${code}`);
  try {
    const result = await axios.get(`${urlPrefix}${code}`, {
      responseType: 'text',
      validateStatus: (c) => c === 200,
      timeout: parseInt(process.env.TIMEOUT) || 10000,
    });
    if (!result.data?.includes('<title>Create VPS &mdash; CLOUDCONE</title>')) {
      console.log(`Bad ${n}-${code}`);
    } else {
      const data = {
        status: result.status,
        headers: result.headers,
        data: result.data,
      };
      console.log(`Good ${n}-${code}: ${JSON.stringify(data)}`);
      await fs.promises.writeFile(
        `data/$${code}.json`,
        JSON.stringify(data, null, 2),
      );
      return true;
    }
  } catch (e) {
    console.error(`Failed ${code}: ${e.message}`);
  }
  return false;
};

async function main() {
  while (current <= max) {
    const start = current;
    const end = Math.min(current + parallel, max + 1);
    console.log(`Running from ${start} to ${end - 1}`);
    const ranges = _.range(start, end);
    const results = await Promise.all(ranges.map(useCode));
    if (results.some((result) => result)) {
      break;
    }
    current = end;
  }
}
main();
